"use strict";
/*
 * @Author       : 邱狮杰
 * @Date         : 2021-06-24 12:51:33
 * @LastEditTime: 2021-06-26 00:14:04
 * @FilePath: /you-will-like/src/core/search/search.ts
 * @Description  :
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SearchHandler = void 0;
const searchBase_1 = require("./searchBase");
class SearchHandler extends searchBase_1.searchBase {
    constructor(initParams) {
        super();
        this.database = initParams || {};
        this.handlerList = new Map();
    }
    /**
     * @description 初始化参数
     * @param { T } val
     */
    initialization(val) {
        if (typeof val === 'function') {
            this.database = val();
            return this;
        }
        this.database = val;
        return this;
    }
    /**
     * @description 清空字段值
     * @param { T } val 可选 ， val不为空则赋值给参数
     */
    resetField(val) {
        if (val) {
            this.database = val;
            return this;
        }
        this.database = {};
        return this;
    }
    hasField(key) {
        return this.has(this.database, key);
    }
    /**
     * @description 注册自定义逻辑
     * @param key
     * @param fn
     * @returns
     */
    registerHandler(key, fn) {
        var _a, _b;
        if ((_a = this === null || this === void 0 ? void 0 : this.handlerList) === null || _a === void 0 ? void 0 : _a.has(key))
            return this;
        (_b = this === null || this === void 0 ? void 0 : this.handlerList) === null || _b === void 0 ? void 0 : _b.set(key, fn);
        return this;
    }
    /**
     * @description 执行逻辑
     * @param key
     * @param val
     * @returns
     */
    transferHandler(key, val) {
        var _a, _b;
        if (!((_a = this === null || this === void 0 ? void 0 : this.handlerList) === null || _a === void 0 ? void 0 : _a.has(key)))
            throw new Error(`您并没有定义${key}相关处理函数`);
        (_b = this.handlerList.get(key)) === null || _b === void 0 ? void 0 : _b({
            add: this.addNewField.bind(this),
            edit: this.rewriteField.bind(this),
            remove: this.deleteField.bind(this),
            has: this.has.bind(this),
            value: val,
        });
        return this;
    }
    /**
     * @description 重写字段
     * @param key
     * @param val
     * @returns
     */
    rewriteField(key, val) {
        if (!Reflect.has(this.database, key))
            return this;
        Reflect.set(this.database, key, val);
        return this;
    }
    /**
     * @description 新增字段
     * @param key
     * @param val
     * @returns
     */
    addNewField(key, val) {
        if (!Reflect.has(this.database, key)) {
            Reflect.set(this.database, key, val);
        }
        return this;
    }
    /**
     * @description 删除字段
     * @param key
     * @returns
     */
    deleteField(key) {
        if (!Reflect.has(this.database, key))
            return this;
        Reflect.deleteProperty(this.database, key);
        return this;
    }
    returnParams() {
        return this.database;
    }
}
exports.SearchHandler = SearchHandler;
