import { ChainOfResponsibilityTypes } from './type'

export function StopBackwardExecution<T>(retVal?: T): T | undefined {
  this.stopDuty(retVal)
  return retVal
}
/**
 * @description 责任链
 */
export class ChainOfResponsibility<T> implements ChainOfResponsibilityTypes {
  private taskList: Function[] | undefined
  private parameter: T | undefined
  private customReturnValue: unknown // 终止责任链时的返回值,可有可无
  private nextParams: unknown
  constructor(fn: Function, params?: T) {
    this.taskList = []
    this.taskList.push(fn)
    this.parameter = params
  }

  dispatch<R>(params?: T) {
    if (params) this.editParams(params)
    this.loopExecution()
    return this.customReturnValue || this.nextParams
  }
  private loopExecution<R>(rewrite?: R | T): void {
    // 使用了StopBackwardExecution函数后不在继续执行
    if (this.customReturnValue) return
    const execution = this.taskList?.shift()?.call(this, rewrite || this.parameter)
    this.nextParams = execution
    // 此处的判断是判断函数中是否使用中断函数
    if (this.customReturnValue) return
    // 真值 且 没有中断
    if (!this.customReturnValue && this.taskList?.length) {
      this.loopExecution(this.nextParams || this.parameter)
    }
  }
  /**
   * @description 停止职责
   */
  private stopDuty(retVal: unknown) {
    this.customReturnValue = retVal
  }

  addChain(fn: Function) {
    this.taskList?.push(fn)
    return this
  }
  editParams(params: T) {
    this.parameter = params
  }
}
