import { ChainOfResponsibility, StopBackwardExecution } from './chainOfResponsibility'

test('ChainOfResponsibility 获取第一个职责返回值', () => {
  function A() {
    return true
  }
  const run = new ChainOfResponsibility(A)
  expect(run.dispatch()).toStrictEqual(true)
})

test('ChainOfResponsibility dispatch只能获取第一个职责返回值', () => {
  function A() {
    return true
  }
  function B() {
    return false
  }
  const run = new ChainOfResponsibility(A).addChain(B)
  expect(run.dispatch()).toStrictEqual(false)
})

test('ChainOfResponsibility 如果你想及时结束一个职责并获取返回值', () => {
  function A() {
    if (1 === 1) return StopBackwardExecution.call(this, 'end!')
    return true
  }
  function B() {
    return false
  }
  const run = new ChainOfResponsibility(A).addChain(B)
  expect(run.dispatch()).toStrictEqual('end!')
})

test('ChainOfResponsibility 如果你想指定所有职责的形参', () => {
  function A(params: number) {
    return params
  }
  function B(params: number) {
    if (1 === 1) return StopBackwardExecution.call(this, params)
    return false
  }
  const run = new ChainOfResponsibility(A).addChain(B)
  const handler = expect(run.dispatch<number>(1))
  handler.toStrictEqual(1)
})
